//Quiz.java presents UM quiz to a student on client machine
//Interacts with server through an instance of QuizClient,
//  which supplies password information and quiz questions,
//  and also records scores in a simple database maintained by QuizServer.
//The GUI for this program uses radio buttons and look and feel from  Swing 1.1
  
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;

public class Quiz extends JFrame //implements ItemListener 
{
   private static Container contentPane;  //returned by getContentPane();
   private Label questionLabel;
   private 	ButtonGroup radioButtons;  
   private JRadioButton radio1, radio2, radio3, radio4;
   private String question, answer1, answer2, answer3, answer4, rightAnswer;
   
   private Socket client;                //network connection to server
   BufferedReader quizInput;        //get quiz questions and answers from file on server
   DataOutputStream quizOutput; //report quiz results to server
   private StringBuffer userAnswers;     //record results of quiz in local client until student is done
   private int totalQuestions,totalCorrect;  //figure out student's score
   
   public Quiz(String serverName)
   {
     super ( "Quiz for The Universal Machine" );
     addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	});
     setBounds(50,200,700,150);         //center the Quiz on screen
     contentPane = getContentPane();
     contentPane.setLayout(new GridLayout(5, 1));   //control layout of questions and Checkbox buttons
             
     openFilesThruSocket(serverName);  //open network connection to QuizServer 
     Passwd p = new Passwd(this);  
   }

   public void openFilesThruSocket(String serverName)
   {  try 
      { client = new Socket( InetAddress.getByName(serverName), 5000 );  
        //quizInput = new DataInputStream( client.getInputStream() );  -- now deprecated
        quizInput = new BufferedReader( new InputStreamReader ( client.getInputStream() ) );
        quizOutput = new DataOutputStream( client.getOutputStream() );
      }
      catch ( IOException e ) 
      { e.printStackTrace(); }
   }

   public void runQuiz()    
   {
      question = new String();
      answer1 = new String();
      answer2 = new String();
      answer3 = new String();
      answer4 = new String();
      rightAnswer = new String();
      userAnswers = new StringBuffer();
    
      try //open file and read questions
      {                    
         question = quizInput.readLine(); //gobble up newline
         question = quizInput.readLine().substring(2);
         answer1 = quizInput.readLine().substring(2);
         answer2 = quizInput.readLine().substring(2);
         answer3 = quizInput.readLine().substring(2);
         answer4 = quizInput.readLine().substring(2);
         rightAnswer = quizInput.readLine().substring(2);
      }
      catch ( IOException e )
      { System.err.println("Error while getting quiz data from server\n" + e.toString() );
        System.exit(1);
      }
       
      //Create Label posing a question
      questionLabel = new Label( question );
      Font questionFont = new Font ( "Arial",Font.BOLD,14 );
      questionLabel.setFont( questionFont );
      contentPane.add( questionLabel ); 
      
       //Create radio buttons and listen for one to get clicked
      RadioListener radioListener = new RadioListener();
      radio1 = new JRadioButton(answer1 , false );
      radio1.addActionListener( radioListener );  
      radio2 = new JRadioButton( answer2, false );
      radio2.addActionListener( radioListener );  
      radio3 = new JRadioButton( answer3, false );
      radio3.addActionListener( radioListener );  
      radio4 = new JRadioButton( answer4, false );
      radio4.addActionListener( radioListener );  
 
      //Create ButtonGroup to maintain radio buttons
      radioButtons = new ButtonGroup();
      radioButtons.add( radio1 );  radioButtons.add( radio2 );   
      radioButtons.add( radio3 );  radioButtons.add( radio4 );
      
      //put radio buttons in this Quiz frame
      contentPane.add(radio1); contentPane.add(radio2); 
      contentPane.add(radio3); contentPane.add(radio4);
      //make everything visible
      pack();
      setVisible( true );
      if (answer4.equals(" ")) radio4.setVisible(false);
   }

   //An ActionListener that listens to the radio buttons
   class RadioListener implements ActionListener 
   {
      public void actionPerformed(ActionEvent ev)
      {  
        String userAnswer = new String();
        if (ev.getSource() == radio1)  { userAnswer = "1 "; radio1.setSelected(false); }
        else if (ev.getSource() == radio2)  { userAnswer = "2 "; radio2.setSelected(false); }
        else if (ev.getSource() == radio3)  { userAnswer = "3 "; radio3.setSelected(false); }
        else if (ev.getSource() == radio4)  { userAnswer = "4 "; radio4.setSelected(false); }
      
        userAnswers.append(userAnswer);
        if (userAnswer.equals(rightAnswer))                         
          totalCorrect++;
        totalQuestions++;
      
        try //read answer and next set  questions
        { 
           question = quizInput.readLine().substring(2);   
           if (question.equals("# "))
              finishQuiz();
           answer1 = quizInput.readLine().substring(2);
           answer2 = quizInput.readLine().substring(2);
           answer3 = quizInput.readLine().substring(2);
           answer4 = quizInput.readLine().substring(2);
           rightAnswer = quizInput.readLine().substring(2);
         }
         catch ( IOException e )
         { System.err.println("Error while reading quiz.dat\n" + e.toString() );
           System.exit(1);
         }
          
        questionLabel.setText(question); 
        radio1.setText(answer1); radio1.setSelected(false);
        radio2.setText(answer2); radio2.setSelected(false);
        radio3.setText(answer3); radio3.setSelected(false);
        radio4.setText(answer4); radio4.setSelected(false);
      
        if (answer4.equals(" ")) 
            radio4.setVisible(false);
        else radio4.setVisible(true);
        pack();
        repaint(); 
     }
   }

   private void finishQuiz() 
   {  
       radio1.setVisible(false);
       radio2.setVisible(false);
       radio3.setVisible(false);
       radio4.setVisible(false);
       question = "You got " + totalCorrect + " out of " + totalQuestions +" right!";
             questionLabel.setText(question); 
                        
      try 
       { quizOutput.writeUTF("score=" + totalCorrect +"\n");
       } catch ( IOException e ) {}              
     
      repaint();
      long startTimer = System.currentTimeMillis();
      while (System.currentTimeMillis() < startTimer + 5000) ;
      try 
      { client.close(); } 
      catch ( IOException e ) {} 
      System.exit(0);
    }

   public class CloseQuizWindow extends WindowAdapter 
   {  public void windowClosing( WindowEvent e )
      { System.out.println("UM quiz aborted... no score recorded...");
        System.exit( 0 );
      }
   }//CloseQuizWindow

   public static void main(String args[])
   { 
     //Check command-line parameter (should be server name)
     if (args.length < 1 || args.length > 1)
     { System.out.println("Usage: java Quiz <server name>");
        System.exit(1);
     } 
   Quiz q = new Quiz(args[0]   );
   q.runQuiz();
    }

}
